"""Top-level package for Python ARTMAP."""

__author__ = """David Vinicius"""
__email__ = 'de.175@hotmail.com'
__version__ = '0.1.0'

from .python_artmap import *
from .art import ART
from .art_fuzzy import ARTFUZZY
from .artmap_fuzzy import ARTMAPFUZZY
